@@ -49,11 +49,16 @@
 
 package com.openexchange.mobile.api.facade.endpoints.responses;
 
+import java.util.List;
 import java.util.Map;
 
 import com.fasterxml.jackson.annotation.JsonInclude;
 import com.fasterxml.jackson.annotation.JsonInclude.Include;
 import com.openexchange.mobile.api.facade.auth.SessionData;
+import com.openexchange.mobile.api.facade.endpoints.responses.ctos.CapabilityCto;
+import com.openexchange.mobile.api.facade.models.Capability;
+import com.openexchange.mobile.api.facade.utils.ListUtil;
+import com.openexchange.mobile.api.facade.utils.MapFunction;
 
 import lombok.AccessLevel;
 import lombok.AllArgsConstructor;
@@ -65,14 +70,26 @@ import lombok.Value;
 @JsonInclude(Include.NON_EMPTY)
 public class AuthenticationResponseBody {
 
+    private static final MapFunction<Capability, CapabilityCto> capabilitiesMapFunction = new MapFunction<Capability, CapabilityCto>() {
+
+        @Override
+        public CapabilityCto map(Capability capability) {
+            return new CapabilityCto(capability);
+        }
+
+    };
+
     @NonNull
     String session;
 
     Map<String, String> properties;
 
+    List<CapabilityCto> capabilities;
+
     public AuthenticationResponseBody(SessionData sessionData) {
         session = sessionData.getSession();
         properties = sessionData.getProperties();
+        capabilities = ListUtil.map(sessionData.getCapabilities(), capabilitiesMapFunction);
     }
 
 }
