@@ -49,14 +49,17 @@
 
 package com.openexchange.mobile.api.facade.exceptions;
 
+import java.util.Collections;
+import java.util.List;
+
 import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.Response;
 import javax.ws.rs.core.Response.Status;
 
+import com.openexchange.java.Strings;
 import com.openexchange.mobile.api.facade.connectors.AbstractResponseMto;
 import com.openexchange.mobile.api.facade.endpoints.responses.AbstractResponseBody;
 import com.openexchange.mobile.api.facade.utils.JacksonUtil;
-import com.openexchange.mobile.api.facade.utils.StringUtils;
 
 import lombok.Getter;
 
@@ -79,6 +82,18 @@ public class ApiFacadeException extends WebApplicationException {
 
         int ACCOUNT_NOT_FOUND = 7;
 
+        int EMPTY_ARRAY_NOT_ALLOWED = 8;
+
+        int INVALID_ACCOUNT_DATA = 9;
+
+        int REDIRECT = 10;
+
+        int SERVICE_UNAVAILABLE = 11;
+
+        int PASSWORD_OF_SECONDARY_MAIL_ACCOUNT_CHANGED = 12;
+
+        int PERMISSION_DENIED = 13;
+
         String INVALID_SESSION = "SES-206";
 
         int CLIENT_UPDATE_MANDATORY = 900;
@@ -104,8 +119,28 @@ public class ApiFacadeException extends WebApplicationException {
             return invalidSession();
         } else if (areCredentialsInvalid(code)) {
             return invalidCredentials();
+        } else if (isDatabaseUpdate(code)) {
+            return serviceUnavailable();
+        } else if (isPasswordOfSecondaryAccountChanged(code)) {
+            return passwordSecondaryAccountChanged();
+        } else if (isPermissionDenied(code)) {
+            return permissionDenied(error);
+        } else if (isRedirect(code)) {
+            return redirect(error);
+        }
+        List<String> categories = getCategories(responseBody.getCategories());
+        return create(Status.BAD_REQUEST, code, null, responseBody.getError(), categories);
+    }
+
+    @SuppressWarnings("unchecked")
+    private static List<String> getCategories(Object categories) {
+        if (categories instanceof String) {
+            return Collections.<String>singletonList((String) categories);
+        } else if (categories instanceof List) {
+            return (List<String>) categories;
+        } else {
+            return null;
         }
-        return create(Status.BAD_REQUEST, code, responseBody.getErrorDescription());
     }
 
     private static boolean isMaximumNumberOfSessionsExceeded(String code) {
@@ -113,14 +148,30 @@ public class ApiFacadeException extends WebApplicationException {
     }
 
     private static boolean isSessionInvalid(String code, String error) {
-        return (!StringUtils.isEmpty(code) && code.startsWith("SES-"))
-                || (!StringUtils.isEmpty(error) && error.startsWith("SES-"));
+        return (!Strings.isEmpty(code) && code.startsWith("SES-"))
+                || (!Strings.isEmpty(error) && error.startsWith("SES-"));
     }
 
     private static boolean areCredentialsInvalid(String code) {
         return "MSG-1000".equals(code) || "LGI-0006".equals(code);
     }
 
+    private static boolean isDatabaseUpdate(String code) {
+        return "CTX-0007".equals(code);
+    }
+
+    private static boolean isPasswordOfSecondaryAccountChanged(String code) {
+        return "MSG-1016".equals(code);
+    }
+
+    private static boolean isPermissionDenied(String code) {
+        return "LGI-0013".equals(code);
+    }
+
+    private static boolean isRedirect(String code) {
+        return "LGI-0016".equals(code);
+    }
+
     public static ApiFacadeException badRequest(int errorCode, String message) {
         return create(Status.BAD_REQUEST, errorCode, message);
     }
@@ -137,6 +188,10 @@ public class ApiFacadeException extends WebApplicationException {
         return create(Status.UNAUTHORIZED, Code.INVALID_SESSION, "Your session was invalidated. Please try again.");
     }
 
+    public static ApiFacadeException redirect(String location) {
+        return create(Status.TEMPORARY_REDIRECT, Code.REDIRECT, location);
+    }
+
     public static ApiFacadeException missingHeader(String name) {
         return create(Status.BAD_REQUEST, Code.MISSING_HEADER, "Header missing: " + name);
     }
@@ -149,14 +204,26 @@ public class ApiFacadeException extends WebApplicationException {
         return create(Status.BAD_REQUEST, Code.INVALID_USER_AGENT, "Invalid User-Agent header");
     }
 
+    public static ApiFacadeException emptyArrayNotAllowed(String name) {
+        return create(Status.BAD_REQUEST, Code.EMPTY_ARRAY_NOT_ALLOWED, "Empty array not allowed: " + name);
+    }
+
     public static ApiFacadeException authenticationNotFound() {
         return internalServerError("Authentication mechanism not found");
     }
 
+    public static ApiFacadeException forbidden() {
+        return create(Status.FORBIDDEN, null, "You are not allowed to access this server");
+    }
+
     public static ApiFacadeException internalServerError(String message) {
         return create(Status.INTERNAL_SERVER_ERROR, Code.INTERNAL_SERVER_ERROR, message);
     }
 
+    public static ApiFacadeException gatewayTimeoutError(Exception cause) {
+        return create(Status.GATEWAY_TIMEOUT, Code.INTERNAL_SERVER_ERROR, cause, "Timeout connecting to middleware");
+    }
+
     public static ApiFacadeException internalServerError(Exception cause) {
         if (cause instanceof ApiFacadeException) {
             return (ApiFacadeException) cause;
@@ -174,15 +241,15 @@ public class ApiFacadeException extends WebApplicationException {
     }
 
     private static ApiFacadeException create(Status status, int errorCode, Exception cause, String message) {
-        return create(status, String.valueOf(errorCode), cause, message);
+        return create(status, String.valueOf(errorCode), cause, message, null);
     }
 
     private static ApiFacadeException create(Status status, String errorCode, String message) {
-        return create(status, errorCode, null, message);
+        return create(status, errorCode, null, message, null);
     }
 
-    private static ApiFacadeException create(Status status, String errorCode, Exception cause, String message) {
-        String responseBody = JacksonUtil.toString(new AbstractResponseBody(errorCode, message));
+    private static ApiFacadeException create(Status status, String errorCode, Exception cause, String message, List<String> categories) {
+        String responseBody = JacksonUtil.toString(new AbstractResponseBody(errorCode, message, categories));
         Response response = Response.status(status).entity(responseBody).build();
         return new ApiFacadeException(errorCode, cause, message, response);
     }
@@ -195,4 +262,20 @@ public class ApiFacadeException extends WebApplicationException {
         return create(Status.NOT_FOUND, Code.ACCOUNT_NOT_FOUND, "Account not found: " + accountId);
     }
 
+    public static ApiFacadeException invalidAccountData() {
+        return create(Status.BAD_REQUEST, Code.INVALID_ACCOUNT_DATA, "Invalid account data");
+    }
+
+    public static ApiFacadeException serviceUnavailable() {
+        return create(Status.SERVICE_UNAVAILABLE, Code.SERVICE_UNAVAILABLE, "Service unavailable");
+    }
+
+    public static ApiFacadeException passwordSecondaryAccountChanged() {
+        return create(Status.SERVICE_UNAVAILABLE, Code.PASSWORD_OF_SECONDARY_MAIL_ACCOUNT_CHANGED, "Password of secondary mail account changed");
+    }
+
+    public static ApiFacadeException permissionDenied(String errorMessage) {
+        return create(Status.FORBIDDEN, Code.PERMISSION_DENIED, errorMessage != null ? errorMessage : "Permission denied");
+    }
+
 }
