@@ -151,7 +151,7 @@ public class Mail {
         this.parts = null;
     }
 
-    private Mail(String id, int flags, int color, Long receivedDate, List<MailContact> from, List<MailContact> to, List<MailContact> cc, List<MailContact> bcc, String subject, String teaser, Long sentDate) {
+    private Mail(String id, int flags, int color, Long receivedDate, List<MailContact> from, List<MailContact> to, List<MailContact> cc, List<MailContact> bcc, String subject, String teaser, Long sentDate, Integer priority, Boolean hasAttachments) {
         this.id = id;
         this.flags = new MailFlags(flags);
         this.color = nonZero(color);
@@ -169,9 +169,9 @@ public class Mail {
         this.accountName = null;
         this.accountId = null;
         this.contentType = null;
-        this.hasAttachments = null;
+        this.hasAttachments = hasAttachments;
         this.size = null;
-        this.priority = null;
+        this.priority = Priority.valueOf(priority);
         this.parts = null;
     }
 
@@ -188,7 +188,7 @@ public class Mail {
         this.bcc = MailContactUtil.convert(mailMto.getBcc());
         this.subject = mailMto.getSubject();
         Map<String, Object> headers = getHeaders(mailMto);
-        this.messageId = (String) headers.get("Message-ID");
+        this.messageId = getString(headers.get("Message-ID"));
         this.replyTo = MailUtil.parseContacts((String) headers.get("Reply-To"));
         this.accountName = mailMto.getAccountName();
         this.accountId = mailMto.getAccountId();
@@ -211,8 +211,8 @@ public class Mail {
         return new Mail(id, flags, color, receivedDate, level);
     }
 
-    public static Mail createEnvelope(String id, int flags, int color, Long receivedDate, List<MailContact> from, List<MailContact> to, List<MailContact> cc, List<MailContact> bcc, String subject, String teaser, Long sentDate) {
-        return new Mail(id, flags, color, receivedDate, from, to, cc, bcc, subject, teaser, sentDate);
+    public static Mail createEnvelope(String id, int flags, int color, Long receivedDate, List<MailContact> from, List<MailContact> to, List<MailContact> cc, List<MailContact> bcc, String subject, String teaser, Long sentDate, Integer priority, Boolean hasAttachments) {
+        return new Mail(id, flags, color, receivedDate, from, to, cc, bcc, subject, teaser, sentDate, priority, hasAttachments);
     }
 
     public static Mail createMail(MailMto mailMto) {
@@ -224,6 +224,18 @@ public class Mail {
         return headers != null ? headers : Collections.<String, Object>emptyMap();
     }
 
+    private String getString(Object object) {
+        if (object instanceof String) {
+            return (String) object;
+        } else if (object instanceof List) {
+            return (String) ((List<?>) object).get(0);
+        } else if (object == null) {
+            return null;
+        } else {
+            throw new RuntimeException("Unsupported type to map to String: " + object.getClass().getSimpleName());
+        }
+    }
+
     private Integer nonZero(int value) {
         return value == 0 ? null : value;
     }
