@@ -51,6 +51,8 @@ package com.openexchange.mobile.api.facade.utils;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 import com.openexchange.mobile.api.facade.models.MailContact;
 
@@ -59,6 +61,10 @@ import lombok.experimental.UtilityClass;
 @UtilityClass
 public class MailContactUtil {
 
+    private static final Pattern PATTERN_BEGINS_WITH_QUOTE = Pattern.compile("^[\\\"'\\\\\\s]");
+
+    private static final Pattern PATTERN_REORDER_DISPLAY_NAME = Pattern.compile("^([^,.\\(\\)]+),\\s([^,.\\(\\)]+)$");
+
     public List<MailContact> convert(List<List<String>> rawList) {
         if (ListUtil.isEmpty(rawList)) {
             return null;
@@ -73,6 +79,9 @@ public class MailContactUtil {
 
             String name = sanitizeName(entryList.get(0));
             String address = entryList.get(1);
+            if (name != null) {
+                name = reorderDisplayName(name);
+            }
             if (address != null && address.contains("@")) {
                 MailContact mailContact = new MailContact(name, address);
                 mailContacts.add(mailContact);
@@ -84,14 +93,32 @@ public class MailContactUtil {
 
     public String sanitizeName(String name) {
         if (name != null) {
-            int length = name.length();
-            if (length > 2
-                    && name.charAt(0) == '"'
-                    && name.charAt(length - 1) == '"') {
-                name = name.substring(1, length - 1);
+            // Remove outer quotes.
+            while (name.length() > 1
+                    && PATTERN_BEGINS_WITH_QUOTE.matcher(name).lookingAt()
+                    && name.charAt(0) == name.charAt(name.length() - 1)) {
+                name = name.substring(1, name.length() - 1).trim();
+            }
+
+            // Unescape inner quotes.
+            name = name.replaceAll("\\\"", "\"");
+
+            // Unescape escaped backslashes.
+            name = name.replaceAll("\\{2}", "\\");
+
+            if (name.length() == 0) {
+                name = null;
             }
         }
         return name;
     }
 
+    public String reorderDisplayName(String name) {
+        Matcher matcher = PATTERN_REORDER_DISPLAY_NAME.matcher(name);
+        if (matcher.matches()) {
+            name = matcher.group(2) + " " + matcher.group(1);
+        }
+        return name;
+    }
+
 }
